
#include "inc.h"
#include "gbfs.h"
#include "lib.h"
#include "minix.h"

#include "tk80.h"
#include "i8255.h"
#include "i8080.h"

//---------------------------------------------------------------------------
//IyR[he[ȕ
ST_I8080_OP_TABLE I8080OpTbl[256] = {
	{ (void*)I8080Nop   },		//00:NOP
	{ (void*)I8080Lxi   },		//01:LXI B
	{ (void*)I8080Stax  },		//02:STAX B
	{ (void*)I8080Inx   },		//03:INX B
	{ (void*)I8080Inr   },		//04:INR B
	{ (void*)I8080Dcr   },		//05:DCR B
	{ (void*)I8080Mvi   },		//06:MVI B
	{ (void*)I8080Rlc   },		//07:RLC
	{ (void*)I8080Undef },		//08:
	{ (void*)I8080Dad   },		//09:DAD B
	{ (void*)I8080Ldax  },		//0A:LDAX B
	{ (void*)I8080Dcx   },		//0B:DCX B
	{ (void*)I8080Inr   },		//0C:INR C
	{ (void*)I8080Dcr   },		//0D:DCR C
	{ (void*)I8080Mvi   },		//0E:MVI C
	{ (void*)I8080Rrc   },		//0F:RRC
	{ (void*)I8080Undef },		//10:
	{ (void*)I8080Lxi   },		//11:LXI D
	{ (void*)I8080Stax  },		//12:STAX D
	{ (void*)I8080Inx   },		//13:INX D
	{ (void*)I8080Inr   },		//14:INR D
	{ (void*)I8080Dcr   },		//15:DCR D
	{ (void*)I8080Mvi   },		//16:MVI D
	{ (void*)I8080Ral   },		//17:RAL
	{ (void*)I8080Undef },		//18:
	{ (void*)I8080Dad   },		//19:DAD D
	{ (void*)I8080Ldax  },		//1A:LDAX D
	{ (void*)I8080Dcx   },		//1B:DCX D
	{ (void*)I8080Inr   },		//1C:INR E
	{ (void*)I8080Dcr   },		//1D:DCR E
	{ (void*)I8080Mvi   },		//1E:MVI E
	{ (void*)I8080Rar   },		//1F:RAR
	{ (void*)I8080Undef },		//20:
	{ (void*)I8080Lxi   },		//21:LXI H
	{ (void*)I8080Shld  },		//22:SHLD
	{ (void*)I8080Inx   },		//23:INX H
	{ (void*)I8080Inr   },		//24:INR H
	{ (void*)I8080Dcr   },		//25:DCR H
	{ (void*)I8080Mvi   },		//26:MVI H
	{ (void*)I8080Daa   },		//27:DAA
	{ (void*)I8080Undef },		//28:
	{ (void*)I8080Dad   },		//29:DAD H
	{ (void*)I8080Lhld  },		//2A:LHLD
	{ (void*)I8080Dcx   },		//2B:DCX H
	{ (void*)I8080Inr   },		//2C:INR L
	{ (void*)I8080Dcr   },		//2D:DCR L
	{ (void*)I8080Mvi   },		//2E:MVI L
	{ (void*)I8080Cma   },		//2F:CMA
	{ (void*)I8080Undef },		//30:
	{ (void*)I8080Lxi   },		//31:LXI SP
	{ (void*)I8080Sta   },		//32:STA
	{ (void*)I8080Inx   },		//33:INX SP
	{ (void*)I8080Inr   },		//34:INR M
	{ (void*)I8080Dcr   },		//35:DCR M
	{ (void*)I8080Mvi   },		//36:MVI M
	{ (void*)I8080Stc   },		//37:STC
	{ (void*)I8080Undef },		//38:
	{ (void*)I8080Dad   },		//39:DAD SP
	{ (void*)I8080Lda   },		//3A:LDA
	{ (void*)I8080Dcx   },		//3B:DCX SP
	{ (void*)I8080Inr   },		//3C:INR A
	{ (void*)I8080Dcr   },		//3D:DCR A
	{ (void*)I8080Mvi   },		//3E:MVI A
	{ (void*)I8080Cmc   },		//3F:CMC
	{ (void*)I8080Mov   },		//40:MOV B,B
	{ (void*)I8080Mov   },		//41:MOV B,C
	{ (void*)I8080Mov   },		//42:MOV B,D
	{ (void*)I8080Mov   },		//43:MOV B,E
	{ (void*)I8080Mov   },		//44:MOV B,H
	{ (void*)I8080Mov   },		//45:MOV B,L
	{ (void*)I8080Mov   },		//46:MOV B,M
	{ (void*)I8080Mov   },		//47:MOV B,A
	{ (void*)I8080Mov   },		//48:MOV C,B
	{ (void*)I8080Mov   },		//49:MOV C,C
	{ (void*)I8080Mov   },		//4A:MOV C,D
	{ (void*)I8080Mov   },		//4B:MOV C,E
	{ (void*)I8080Mov   },		//4C:MOV C,H
	{ (void*)I8080Mov   },		//4D:MOV C,L
	{ (void*)I8080Mov   },		//4E:MOV C,M
	{ (void*)I8080Mov   },		//4F:MOV C,A
	{ (void*)I8080Mov   },		//50:MOV D,B
	{ (void*)I8080Mov   },		//51:MOV D,C
	{ (void*)I8080Mov   },		//52:MOV D,D
	{ (void*)I8080Mov   },		//53:MOV D,E
	{ (void*)I8080Mov   },		//54:MOV D,H
	{ (void*)I8080Mov   },		//55:MOV D,L
	{ (void*)I8080Mov   },		//56:MOV D,M
	{ (void*)I8080Mov   },		//57:MOV D,A
	{ (void*)I8080Mov   },		//58:MOV E,B
	{ (void*)I8080Mov   },		//59:MOV E,C
	{ (void*)I8080Mov   },		//5A:MOV E,D
	{ (void*)I8080Mov   },		//5B:MOV E,E
	{ (void*)I8080Mov   },		//5C:MOV E,H
	{ (void*)I8080Mov   },		//5D:MOV E,L
	{ (void*)I8080Mov   },		//5E:MOV E,M
	{ (void*)I8080Mov   },		//5F:MOV E,A
	{ (void*)I8080Mov   },		//60:MOV H,B
	{ (void*)I8080Mov   },		//61:MOV H,C
	{ (void*)I8080Mov   },		//62:MOV H,D
	{ (void*)I8080Mov   },		//63:MOV H,E
	{ (void*)I8080Mov   },		//64:MOV H,H
	{ (void*)I8080Mov   },		//65:MOV H,L
	{ (void*)I8080Mov   },		//66:MOV H,M
	{ (void*)I8080Mov   },		//67:MOV H,A
	{ (void*)I8080Mov   },		//68:MOV L,B
	{ (void*)I8080Mov   },		//69:MOV L,C
	{ (void*)I8080Mov   },		//6A:MOV L,D
	{ (void*)I8080Mov   },		//6B:MOV L,E
	{ (void*)I8080Mov   },		//6C:MOV L,H
	{ (void*)I8080Mov   },		//6D:MOV L,L
	{ (void*)I8080Mov   },		//6E:MOV L,M
	{ (void*)I8080Mov   },		//6F:MOV L,A
	{ (void*)I8080Mov   },		//70:MOV M,B
	{ (void*)I8080Mov   },		//71:MOV M,C
	{ (void*)I8080Mov   },		//72:MOV M,D
	{ (void*)I8080Mov   },		//73:MOV M,E
	{ (void*)I8080Mov   },		//74:MOV M,H
	{ (void*)I8080Mov   },		//75:MOV M,L
	{ (void*)I8080Hlt   },		//76:HLT
	{ (void*)I8080Mov   },		//77:MOV M,A
	{ (void*)I8080Mov   },		//78:MOV A,B
	{ (void*)I8080Mov   },		//79:MOV A,C
	{ (void*)I8080Mov   },		//7A:MOV A,D
	{ (void*)I8080Mov   },		//7B:MOV A,E
	{ (void*)I8080Mov   },		//7C:MOV A,H
	{ (void*)I8080Mov   },		//7D:MOV A,L
	{ (void*)I8080Mov   },		//7E:MOV A,M
	{ (void*)I8080Mov   },		//7F:MOV A,A
	{ (void*)I8080Add   },		//80:ADD B
	{ (void*)I8080Add   },		//81:ADD C
	{ (void*)I8080Add   },		//82:ADD D
	{ (void*)I8080Add   },		//83:ADD E
	{ (void*)I8080Add   },		//84:ADD H
	{ (void*)I8080Add   },		//85:ADD L
	{ (void*)I8080Add   },		//86:ADD M
	{ (void*)I8080Add   },		//87:ADD A
	{ (void*)I8080Adc   },		//88:ADC B
	{ (void*)I8080Adc   },		//89:ADC C
	{ (void*)I8080Adc   },		//8A:ADC D
	{ (void*)I8080Adc   },		//8B:ADC E
	{ (void*)I8080Adc   },		//8C:ADC H
	{ (void*)I8080Adc   },		//8D:ADC L
	{ (void*)I8080Adc   },		//8E:ADC M
	{ (void*)I8080Adc   },		//8F:ADC A
	{ (void*)I8080Sub   },		//90:SUB B
	{ (void*)I8080Sub   },		//91:SUB C
	{ (void*)I8080Sub   },		//92:SUB D
	{ (void*)I8080Sub   },		//93:SUB E
	{ (void*)I8080Sub   },		//94:SUB H
	{ (void*)I8080Sub   },		//95:SUB L
	{ (void*)I8080Sub   },		//96:SUB M
	{ (void*)I8080Sub   },		//97:SUB A
	{ (void*)I8080Sbb   },		//98:SBB B
	{ (void*)I8080Sbb   },		//99:SBB C
	{ (void*)I8080Sbb   },		//9A:SBB D
	{ (void*)I8080Sbb   },		//9B:SBB E
	{ (void*)I8080Sbb   },		//9C:SBB H
	{ (void*)I8080Sbb   },		//9D:SBB L
	{ (void*)I8080Sbb   },		//9E:SBB M
	{ (void*)I8080Sbb   },		//9F:SBB A
	{ (void*)I8080Ana   },		//A0:ANA B
	{ (void*)I8080Ana   },		//A1:ANA C
	{ (void*)I8080Ana   },		//A2:ANA D
	{ (void*)I8080Ana   },		//A3:ANA E
	{ (void*)I8080Ana   },		//A4:ANA H
	{ (void*)I8080Ana   },		//A5:ANA L
	{ (void*)I8080Ana   },		//A6:ANA M
	{ (void*)I8080Ana   },		//A7:ANA A
	{ (void*)I8080Xra   },		//A8:XRA B
	{ (void*)I8080Xra   },		//A9:XRA C
	{ (void*)I8080Xra   },		//AA:XRA D
	{ (void*)I8080Xra   },		//AB:XRA E
	{ (void*)I8080Xra   },		//AC:XRA H
	{ (void*)I8080Xra   },		//AD:XRA L
	{ (void*)I8080Xra   },		//AE:XRA M
	{ (void*)I8080Xra   },		//AF:XRA A
	{ (void*)I8080Ora   },		//B0:ORA B
	{ (void*)I8080Ora   },		//B1:ORA C
	{ (void*)I8080Ora   },		//B2:ORA D
	{ (void*)I8080Ora   },		//B3:ORA E
	{ (void*)I8080Ora   },		//B4:ORA H
	{ (void*)I8080Ora   },		//B5:ORA L
	{ (void*)I8080Ora   },		//B6:ORA M
	{ (void*)I8080Ora   },		//B7:ORA A
	{ (void*)I8080Cmp   },		//B8:CMP B
	{ (void*)I8080Cmp   },		//B9:CMP C
	{ (void*)I8080Cmp   },		//BA:CMP D
	{ (void*)I8080Cmp   },		//BB:CMP E
	{ (void*)I8080Cmp   },		//BC:CMP H
	{ (void*)I8080Cmp   },		//BD:CMP L
	{ (void*)I8080Cmp   },		//BE:CMP M
	{ (void*)I8080Cmp   },		//BF:CMP A
	{ (void*)I8080Ret   },		//C0:RNZ
	{ (void*)I8080Pop   },		//C1:POP B
	{ (void*)I8080Jmp   },		//C2:JNZ
	{ (void*)I8080Jmp   },		//C3:JMP
	{ (void*)I8080Call  },		//C4:CNZ
	{ (void*)I8080Push  },		//C5:PUSH B
	{ (void*)I8080Adi   },		//C6:ADI
	{ (void*)I8080Rst   },		//C7:Rst 0
	{ (void*)I8080Ret   },		//C8:RZ
	{ (void*)I8080Ret   },		//C9:RET
	{ (void*)I8080Jmp   },		//CA:JZ
	{ (void*)I8080Undef },		//CB:
	{ (void*)I8080Call  },		//CC:CZ
	{ (void*)I8080Call  },		//CD:CALL
	{ (void*)I8080Adi   },		//CE:ACI
	{ (void*)I8080Rst   },		//CF:Rst 1
	{ (void*)I8080Ret   },		//D0:RNC
	{ (void*)I8080Pop   },		//D1:POP D
	{ (void*)I8080Jmp   },		//D2:JNC
	{ (void*)I8080InOut },		//D3:OUT
	{ (void*)I8080Call  },		//D4:CNC
	{ (void*)I8080Push  },		//D5:PUSH D
	{ (void*)I8080Sbi   },		//D6:SUI
	{ (void*)I8080Rst   },		//D7:Rst 2
	{ (void*)I8080Ret   },		//D8:RC
	{ (void*)I8080Undef },		//D9:
	{ (void*)I8080Jmp   },		//DA:JC
	{ (void*)I8080InOut },		//DB:IN
	{ (void*)I8080Call  },		//DC:CC
	{ (void*)I8080Undef },		//DD:
	{ (void*)I8080Sbi   },		//DE:SBI
	{ (void*)I8080Rst   },		//DF:Rst 3
	{ (void*)I8080Ret   },		//E0:RPO
	{ (void*)I8080Pop   },		//E1:POP H
	{ (void*)I8080Jmp   },		//E2:JPO
	{ (void*)I8080Xthl  },		//E3:XTHL
	{ (void*)I8080Call  },		//E4:CPO
	{ (void*)I8080Push  },		//E5:PUSH H
	{ (void*)I8080Ani   },		//E6:ANI
	{ (void*)I8080Rst   },		//E7:Rst 4
	{ (void*)I8080Ret   },		//E8:RPE
	{ (void*)I8080Pchl  },		//E9:PCHL
	{ (void*)I8080Jmp   },		//EA:JPE
	{ (void*)I8080Xchg  },		//EB:XCHG
	{ (void*)I8080Call  },		//EC:CPE
	{ (void*)I8080Undef },		//ED:
	{ (void*)I8080Xri   },		//EE:XRI
	{ (void*)I8080Rst   },		//EF:Rst 5
	{ (void*)I8080Ret   },		//F0:RP
	{ (void*)I8080Pop   },		//F1:POP PSW
	{ (void*)I8080Jmp   },		//F2:JP
	{ (void*)I8080DiEi  },		//F3:DI
	{ (void*)I8080Call  },		//F4:CP
	{ (void*)I8080Push  },		//F5:PUSH PSW
	{ (void*)I8080Ori   },		//F6:ORI
	{ (void*)I8080Rst   },		//F7:Rst 6
	{ (void*)I8080Ret   },		//F8:RM
	{ (void*)I8080Sphl  },		//F9:SPHL
	{ (void*)I8080Jmp   },		//FA:JM
	{ (void*)I8080DiEi  },		//FB:EI
	{ (void*)I8080Call  },		//FC:CM
	{ (void*)I8080Undef },		//FD:
	{ (void*)I8080Cpi   },		//FE:CPI
	{ (void*)I8080Rst   },		//FF:Rst 7
};
//---------------------------------------------------------------------------
ST_I8080 I8080;

//tk80.c
extern ST_TK80 Tk80;
//---------------------------------------------------------------------------
IWRAM_CODE void I8080Init()
{
	_Memset((u8*)&I8080, 0x00, sizeof(ST_I8080));

	ST_I8080* p = &I8080;

	p->isDaaSub    = TRUE;			//ZDAAs̃tO
	p->isIllInst   = FALSE;			//s߂̃gbv
	p->addrMask    = 0x3ff;			//sANZXop̃AhX}XN
	p->scanRate    = 512;			//bNlƂ̖ߎs
	p->scanIdle    = 0;				//bNlƂ̃ACh

	p->isReset     = TRUE;			//ZbgM̃tO
}
//---------------------------------------------------------------------------
IWRAM_CODE void I8080Reset()
{
	ST_I8080* p = &I8080;

	p->regPC       = 0x0000;
	p->flgPSW      = 0xff;

	p->isIntEnable = FALSE;			//荞݃Cl[uMtO
	p->isIntFF     = FALSE;			//IRQMxpF/FtO
	p->isIntIRQ    = FALSE;			//荞݃NGXgtO

	p->isReset     = FALSE;			//ZbgM̃tO
}
//---------------------------------------------------------------------------
IWRAM_CODE void I8080Exec()
{
	ST_I8080* p = &I8080;
	u8  op;
	u16 i;

	if(p->isReset == TRUE)
	{
		I8080Reset();
		I8255Reset();
	}

	if(p->isSingleStep == TRUE)
	{
		//INTEMxIRQɓ܂
		p->isIntIRQ = p->isIntFF;
		p->isIntFF  = p->isIntEnable;
	}
	else
	{
		p->isIntFF  = FALSE;
		p->isIntIRQ = FALSE;
	}

	//wait
	WaitMs(p->scanIdle);

	for(i=0; i<p->scanRate; i++)
	{
//		I8080Trace();
		op = I8080GetAtPC();
		I8080OpTbl[op].pFunc(op);

		//荞ݏ
		if((p->isIntEnable == TRUE) && (p->isIntIRQ == TRUE))
		{
			TRACEOUT2("Interrupt CPU\n");
			p->isIntEnable = FALSE;

			//RST 7s܂
			I8080Rst(0xff);
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void I8080Trace()
{
	ST_I8080*  p = &I8080;
	u8        op = I8080MemRead(p->regPC);
	char buf[128];

	//REGISTER
	_Sprintf(buf, "A:%02x B:%02x C:%02x D:%02x E:%02x H:%02x L:%02x\n",
		p->reg[I8080_REG_A],
		p->reg[I8080_REG_B],
		p->reg[I8080_REG_C],
		p->reg[I8080_REG_D],
		p->reg[I8080_REG_E],
		p->reg[I8080_REG_H],
		p->reg[I8080_REG_L]);
	TRACEOUT(buf);

	//OP, PC, SP
	_Sprintf(buf, "OP:%02x PC:%04x SP:%04x ", op, p->regPC, p->regSP);
	TRACEOUT(buf);

	//PSW
	_Sprintf(buf, "PSW:%c%c%c%c-%c-%c\n\n",
		(p->flgPSW & I8080_FLG_SIGN)   ? 's' : '-',
		(p->flgPSW & I8080_FLG_ZERO)   ? 'z' : '-',
		(p->flgPSW & I8080_FLG_SUB)    ? 's' : '-',
		(p->flgPSW & I8080_FLG_CY4)    ? '4' : '-',
		(p->flgPSW & I8080_FLG_PARITY) ? 'e' : 'o',
		(p->flgPSW & I8080_FLG_CARRY)  ? 'c' : '-');
	TRACEOUT(buf);
}
//---------------------------------------------------------------------------
IWRAM_CODE u8 I8080MemRead(u16 addr)
{
	ST_I8080* p = &I8080;

	if((p->isReset == TRUE) || (p->isProtect == TRUE))
	{
		return 0xff;
	}

	if(addr >= 0x8000)
	{
		return Tk80.ram[(addr - 0x8000) & p->addrMask];
	}

	return Tk80.rom[addr & p->addrMask];
}
//---------------------------------------------------------------------------
IWRAM_CODE void I8080MemWrite(u16 addr, u8 val)
{
	ST_I8080* p = &I8080;

	if((p->isReset == TRUE) || (p->isProtect == TRUE))
	{
		return;
	}

	if(addr >= 0x8000)
	{
		Tk80.ram[(addr - 0x8000) & p->addrMask] = val;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE u8 I8080GetAtPC()
{
	return I8080MemRead(I8080.regPC++);
}
//---------------------------------------------------------------------------
IWRAM_CODE u16 I8080GetReg16(u8 regH, u8 regL)
{
	ST_I8080* p = &I8080;

	u16 h = (u16)p->reg[regH] << 8;
	u16 l = (u16)p->reg[regL];

	return (h | l);
}
//---------------------------------------------------------------------------
IWRAM_CODE void I8080SetReg16(u8 regH, u8 regL, u16 data)
{
	ST_I8080* p = &I8080;

	p->reg[regH] = (u8)((data >> 8) & 0xff);
	p->reg[regL] = (u8) (data       & 0xff);
}
//---------------------------------------------------------------------------
IWRAM_CODE u8 I8080GetReg8(u8 regNum)
{
	ST_I8080* p = &I8080;

	if(regNum == I8080_REG_M)
	{
		u16 addr = I8080GetReg16(I8080_REG_H, I8080_REG_L);
		return I8080MemRead(addr);
	}

	return p->reg[regNum];
}
//---------------------------------------------------------------------------
IWRAM_CODE void I8080SetReg8(u8 regNum, u8 val)
{
	ST_I8080* p = &I8080;

	if(regNum == I8080_REG_M)
	{
		u16 addr = I8080GetReg16(I8080_REG_H, I8080_REG_L);
		I8080MemWrite(addr, val);

		return;
	}

	p->reg[regNum] = val;
}
//---------------------------------------------------------------------------
//ėp̃OOB̑̃tO̓IyR[hʂɍsĂ܂B
IWRAM_CODE void I8080OpSetFlag(u8 data)
{
	ST_I8080* p = &I8080;

	//Zero flag
	if(data == 0)
	{
		p->flgPSW |=  I8080_FLG_ZERO;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_ZERO;
	}

	//Sign flag
	if (data & 0x80)
	{
		p->flgPSW |=  I8080_FLG_SIGN;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_SIGN;
	}

	//Parity flag
	u16 n = 0;
	u16 i;

	for(i=0; i<8; i++)
	{
		n     += data & 0x01;
		data >>= 1;
	}

	if(n & 0x01)
	{
		p->flgPSW &= ~I8080_FLG_PARITY;
	} else {
		p->flgPSW |=  I8080_FLG_PARITY;
	}
}
//---------------------------------------------------------------------------
//Zʏ
IWRAM_CODE void I8080OpAdd(u16 t1, u16 t2, u16 c)
{
	ST_I8080* p = &I8080;
	u16 sum = t1 + t2 + c;

	I8080SetReg8(I8080_REG_A, sum & 0xff);

	p->flgPSW &= ~I8080_FLG_SUB;
	if(sum & 0x100)
	{
		p->flgPSW |=  I8080_FLG_CARRY;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CARRY;
	}

	if(((t1 & 0x0f) + (t2 & 0x0f) + c) & 0x10)
	{
		p->flgPSW |=  I8080_FLG_CY4;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CY4;
	}

	I8080OpSetFlag(sum & 0xff);
}
//---------------------------------------------------------------------------
//Zʏ
IWRAM_CODE u8 I8080OpSub(u16 t1, u16 t2, u16 c)
{
	ST_I8080* p = &I8080;
	u16 diff = t2 - t1 - c;

	p->flgPSW |= I8080_FLG_SUB;
	if(diff & 0x100)
	{
		p->flgPSW |=  I8080_FLG_CARRY;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CARRY;
	}

	if(((t2 & 0x0f) - (t1 & 0x0f) - c) & 0x10)
	{
		p->flgPSW |=  I8080_FLG_CY4;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CY4;
	}

	I8080OpSetFlag(diff & 0xff);
	return (diff & 0xff);
}
//---------------------------------------------------------------------------
IWRAM_CODE void I8080Undef(u8 op)
{
	TRACEOUT2("Undef:\n");

	ST_I8080* p = &I8080;

	//s߂̃gbṽ`FbN
	if(p->isIllInst == TRUE)
	{
		I8080Rst(0xff);
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void I8080Nop(u8 op)
{
	TRACEOUT2("NOP:\n");
}
//---------------------------------------------------------------------------
//LXI (01,11,21,31)	BC/DE/HL/SP = Imm
IWRAM_CODE void I8080Lxi(u8 op)
{
	ST_I8080* p = &I8080;

	switch(op)
	{
	case 0x01:
		TRACEOUT2("LXI B:\n");
		p->reg[I8080_REG_C] = I8080GetAtPC();
		p->reg[I8080_REG_B] = I8080GetAtPC();
		break;

	case 0x11:
		TRACEOUT2("LXI D:\n");
		p->reg[I8080_REG_E] = I8080GetAtPC();
		p->reg[I8080_REG_D] = I8080GetAtPC();
		break;

	case 0x21:
		TRACEOUT2("LXI H:\n");
		p->reg[I8080_REG_L] = I8080GetAtPC();
		p->reg[I8080_REG_H] = I8080GetAtPC();
		break;

	case 0x31:
		TRACEOUT2("LXI SP:\n");
		p->regSP            = I8080GetAtPC();
		p->regSP           |= I8080GetAtPC() << 8;
		break;

	default:
		TRACEOUT2("LXI Err:\n");
		return;
	}
}
//---------------------------------------------------------------------------
//STAX (02,12)		[BC]/[DE] = A
IWRAM_CODE void I8080Stax(u8 op)
{
	ST_I8080* p = &I8080;
	u16 addr;

	if(op == 0x02)
	{
		TRACEOUT2("STAX B:\n");
		addr = I8080GetReg16(I8080_REG_B, I8080_REG_C);
	}
	else
	{
		TRACEOUT2("STAX D:\n");
		addr = I8080GetReg16(I8080_REG_D, I8080_REG_E);
	}

	I8080MemWrite(addr, p->reg[I8080_REG_A]);
}
//---------------------------------------------------------------------------
//INX (03,13,23,33)	BC/DE/HL/SP++
IWRAM_CODE void I8080Inx(u8 op)
{
	ST_I8080* p = &I8080;

	if(op == 0x33)
	{
		TRACEOUT2("INX SP:\n");
		p->regSP++;

		return;
	}

	u16 data;
	u8  h, l;

	switch (op)
	{
	case 0x03:
		TRACEOUT2("INX B:\n");
		h = I8080_REG_B;
		l = I8080_REG_C;
		break;

	case 0x13:
		TRACEOUT2("INX D:\n");
		h = I8080_REG_D;
		l = I8080_REG_E;
		break;

	case 0x23:
		TRACEOUT2("INX H:\n");
		h = I8080_REG_H;
		l = I8080_REG_L;
		break;

	default:
		TRACEOUT2("INX Err:\n");
		return;
	}

	data = I8080GetReg16(h, l);
	data++;

	I8080SetReg16(h, l, data);
}
//---------------------------------------------------------------------------
//INR (04,0C,14,1C,24,2C,34,3C)	REG/(HL)++
IWRAM_CODE void I8080Inr(u8 op)
{
	ST_I8080* p = &I8080;

	u8 t1 = (op & 0x38) >> 3;
	u8 t2 = I8080GetReg8(t1);

	TRACEOUT2("INR %d:\n", t1);

	t2++;

	I8080SetReg8(t1, t2);
	I8080OpSetFlag(t2);
	p->flgPSW &= ~I8080_FLG_SUB;
}
//---------------------------------------------------------------------------
//DCR (05,0D,15,1D,25,2D,35,3D)	REG/(HL)--
IWRAM_CODE void I8080Dcr(u8 op)
{
	ST_I8080* p = &I8080;

	u8 t1 = (op & 0x38) >> 3;
	u8 t2 = I8080GetReg8(t1);

	TRACEOUT2("DCR %d:\n", t1);

	t2--;

	I8080SetReg8(t1, t2);
	I8080OpSetFlag(t2);
	p->flgPSW |= I8080_FLG_SUB;
}
//---------------------------------------------------------------------------
//RLC (07)
IWRAM_CODE void I8080Rlc(u8 op)
{
	TRACEOUT2("RLC:\n");

	ST_I8080* p = &I8080;
	u16 t = p->reg[I8080_REG_A] << 1;

	if(t & 0x100)
	{
		t |=  1;
		p->flgPSW |=  I8080_FLG_CARRY;
	}
	else
	{
		t &= ~1;
		p->flgPSW &= ~I8080_FLG_CARRY;
	}

	p->reg[I8080_REG_A] = t & 0xff;
}
//---------------------------------------------------------------------------
//RRC (0F)
IWRAM_CODE void I8080Rrc(u8 op)
{
	TRACEOUT2("RRC:\n");

	ST_I8080* p = &I8080;
	u16 t = p->reg[I8080_REG_A];

	if(t & 0x1)
	{
		p->flgPSW |=  I8080_FLG_CARRY;
		t |= 0x100;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CARRY;
	}

	t >>= 1;
	p->reg[I8080_REG_A] = t & 0xff;
}
//---------------------------------------------------------------------------
//RAL (17)
IWRAM_CODE void I8080Ral(u8 op)
{
	TRACEOUT2("RAL:\n");

	ST_I8080* p = &I8080;

	u16 t = p->reg[I8080_REG_A];
	u16 c = t & 0x80;

	t <<= 1;
	if(p->flgPSW & I8080_FLG_CARRY)
	{
		t |= 1;
	}

	if(c != 0x0)
	{
		p->flgPSW |=  I8080_FLG_CARRY;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CARRY;
	}

	p->reg[I8080_REG_A] = t & 0xff;
}
//---------------------------------------------------------------------------
//RAR (1F)
IWRAM_CODE void I8080Rar(u8 op)
{
	TRACEOUT2("RAR:\n");

	ST_I8080* p = &I8080;

	u16 t = p->reg[I8080_REG_A];
	u16 c = t & 0x1;

	t >>= 1;
	if(p->flgPSW & I8080_FLG_CARRY)
	{
		t |= 0x80;
	}

	if(c != 0x0)
	{
		p->flgPSW |=  I8080_FLG_CARRY;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CARRY;
	}

	p->reg[I8080_REG_A] = t & 0xff;
}
//---------------------------------------------------------------------------
//MVI (06,0E,16,1E,26,2E,36,3E)	REG/(HL) = Imm
IWRAM_CODE void I8080Mvi(u8 op)
{
	u8 t = (op & 0x38) >> 3;
	TRACEOUT2("MVI %d:\n", t);

	I8080SetReg8(t, I8080GetAtPC());
}
//---------------------------------------------------------------------------
//DAD (09,19,29,39)	HL += BC/DE/HL/SP
IWRAM_CODE void I8080Dad(u8 op)
{
	ST_I8080* p = &I8080;

	u16 t1, t2;
	u32 sum;

	switch (op)
	{
	case 0x09:
		TRACEOUT2("DAD B:\n");
		t1 = I8080GetReg16(I8080_REG_B, I8080_REG_C);
		break;

	case 0x19:
		TRACEOUT2("DAD D:\n");
		t1 = I8080GetReg16(I8080_REG_D, I8080_REG_E);
		break;

	case 0x29:
		TRACEOUT2("DAD H:\n");
		t1 = I8080GetReg16(I8080_REG_H, I8080_REG_L);
		break;

	case 0x39:
		TRACEOUT2("DAD SP:\n");
		t1 = p->regSP;
		break;

	default:
		TRACEOUT2("DAD Err:\n");
		return;
	}

	t2  = I8080GetReg16(I8080_REG_H, I8080_REG_L);
	sum = (u32)t1 + (u32)t2;
	I8080SetReg16(I8080_REG_H, I8080_REG_L, (u16)sum);


	if(sum & 0x10000)
	{
		p->flgPSW |=  I8080_FLG_CARRY;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CARRY;
	}

	p->flgPSW &= ~I8080_FLG_SUB;

	if(((t1 & 0x0fff) + (t2 & 0x0fff)) & 0x0100)
	{
		p->flgPSW |=  I8080_FLG_CY4;
	}
	else
	{
		p->flgPSW &= ~I8080_FLG_CY4;
	}
}
//---------------------------------------------------------------------------
//LDAX (0A,1A)		A = [BC]/[DE]
IWRAM_CODE void I8080Ldax(u8 op)
{
	ST_I8080* p = &I8080;
	u16 addr;

	if(op == 0x0a)
	{
		TRACEOUT2("LDAX B:\n");
		addr = I8080GetReg16(I8080_REG_B, I8080_REG_C);
	}
	else
	{
		TRACEOUT2("LDAX D:\n");
		addr = I8080GetReg16(I8080_REG_D, I8080_REG_E);
	}

	p->reg[I8080_REG_A] = I8080MemRead(addr);
	return;
}
//---------------------------------------------------------------------------
//DCX (0B,1B,2B,3B)	BC/DE/HL/SP--
IWRAM_CODE void I8080Dcx(u8 op)
{
	ST_I8080* p = &I8080;

	if(op == 0x3b)
	{
		TRACEOUT2("DCX SP:\n");
		p->regSP--;

		return;
	}

	u16 data;
	u8  h, l;

	switch(op)
	{
	case 0x0b:
		TRACEOUT2("DCX B:\n");
		h = I8080_REG_B;
		l = I8080_REG_C;
		break;

	case 0x1b:
		TRACEOUT2("DCX D:\n");
		h = I8080_REG_D;
		l = I8080_REG_E;
		break;

	case 0x2b:
		TRACEOUT2("DCX H:\n");
		h = I8080_REG_H;
		l = I8080_REG_L;
		break;

	default:
		TRACEOUT2("DCX Err:\n");
		return;
	}

	data = I8080GetReg16(h, l);
	data--;

	I8080SetReg16(h, l, data);
}
//---------------------------------------------------------------------------
//STA (32)     [imm] = A
IWRAM_CODE void I8080Sta(u8 op)
{
	TRACEOUT2("STA:\n");

	ST_I8080* p = &I8080;
	u16 addr = I8080GetAtPC() | (I8080GetAtPC() << 8);

	I8080MemWrite(addr, p->reg[I8080_REG_A]);
}
//---------------------------------------------------------------------------
//LDA (3A)     A = [imm]
IWRAM_CODE void I8080Lda(u8 op)
{
	TRACEOUT2("LDA:\n");

	ST_I8080* p = &I8080;
	u16 addr = I8080GetAtPC() | (I8080GetAtPC() << 8);

	p->reg[I8080_REG_A] = I8080MemRead(addr);
}
//---------------------------------------------------------------------------
//SHLD (22)     [imm] = HL
IWRAM_CODE void I8080Shld(u8 op)
{
	TRACEOUT2("SHLD:\n");

	ST_I8080* p = &I8080;
	u16 addr = I8080GetAtPC() | (I8080GetAtPC() << 8);

	I8080MemWrite(addr+0, p->reg[I8080_REG_L]);
	I8080MemWrite(addr+1, p->reg[I8080_REG_H]);
}
//---------------------------------------------------------------------------
//LHLD (2A)     HL = [imm]
IWRAM_CODE void I8080Lhld(u8 op)
{
	TRACEOUT2("LHLD:\n");

	ST_I8080* p = &I8080;
	u16 addr = I8080GetAtPC() | (I8080GetAtPC() << 8);

	p->reg[I8080_REG_L] = I8080MemRead(addr+0);
	p->reg[I8080_REG_H] = I8080MemRead(addr+1);
}
//---------------------------------------------------------------------------
//CMA (2F)      A = NOT A
IWRAM_CODE void I8080Cma(u8 op)
{
	TRACEOUT2("CMA:\n");

	ST_I8080* p = &I8080;
	u8 t = p->reg[I8080_REG_A];

	t = ~t;

	p->reg[I8080_REG_A] = t;
}
//---------------------------------------------------------------------------
//STC (37)      CARRY = 1
IWRAM_CODE void I8080Stc(u8 op)
{
	TRACEOUT2("STC:\n");

	ST_I8080* p = &I8080;
	p->flgPSW |= I8080_FLG_CARRY;
}
//---------------------------------------------------------------------------
//CMC (3F)      CARRY = NOT CARRY
IWRAM_CODE void I8080Cmc(u8 op)
{
	TRACEOUT2("CMC:\n");

	ST_I8080* p = &I8080;
	p->flgPSW ^= I8080_FLG_CARRY;
}
//---------------------------------------------------------------------------
//DAA (27)      Decimal Adjust
IWRAM_CODE void I8080Daa(u8 op)
{
	TRACEOUT2("DAA:\n");

	ST_I8080* p = &I8080;
	u16 a = p->reg[I8080_REG_A];

	if(p->isDaaSub == TRUE && (p->flgPSW & I8080_FLG_SUB) != 0x00)
	{
		//ZiNEC[ĥ݁j
		if(p->flgPSW & I8080_FLG_CY4)
		{
			a -= 6;
		}
		if(p->flgPSW & I8080_FLG_CARRY)
		{
			a -= 0x60;
		}
	}
	else
	{
		//ZiWj
		if(((a & 0x0f) > 9) || (p->flgPSW & I8080_FLG_CY4))
		{
			a += 6;
		}

		if(((a & 0xf0) > 0x90) || (p->flgPSW & I8080_FLG_CARRY))
		{
			a += 0x60;
			p->flgPSW |= I8080_FLG_CARRY;
		}
	}

	p->reg[I8080_REG_A] = (u8)(a & 0xff);
	I8080OpSetFlag((u8)(a & 0xff));
}
//---------------------------------------------------------------------------
//MOV (40-7F(76))
IWRAM_CODE void I8080Mov(u8 op)
{
	u8 src = (op & 0x07);
	u8 dst = (op & 0x38) >> 3;

	TRACEOUT2("MOV %d,%d:\n", dst, src);
	I8080SetReg8(dst, I8080GetReg8(src));
}
//---------------------------------------------------------------------------
//XCHG (EB)     swap HL and DE
IWRAM_CODE void I8080Xchg(u8 op)
{
	TRACEOUT2("XCHG:\n");

	u16 t1 = I8080GetReg16(I8080_REG_H, I8080_REG_L);
	u16 t2 = I8080GetReg16(I8080_REG_D, I8080_REG_E);

	I8080SetReg16(I8080_REG_H, I8080_REG_L, t2);
	I8080SetReg16(I8080_REG_D, I8080_REG_E, t1);
}
//---------------------------------------------------------------------------
//XTHL (E3)     swap HL and [SP]
IWRAM_CODE void I8080Xthl(u8 op)
{
	TRACEOUT2("XTHL:\n");

	ST_I8080* p = &I8080;

	u8 t1 = p->reg[I8080_REG_H];
	u8 t2 = p->reg[I8080_REG_L];

	p->reg[I8080_REG_H] = I8080MemRead(p->regSP+1);
	p->reg[I8080_REG_L] = I8080MemRead(p->regSP+0);

	I8080MemWrite(p->regSP+0, t2);
	I8080MemWrite(p->regSP+1, t1);
}
//---------------------------------------------------------------------------
//SPHL (F1)     SP = HL
IWRAM_CODE void I8080Sphl(u8 op)
{
	TRACEOUT2("SPHL:\n");

	ST_I8080* p = &I8080;
	p->regSP = I8080GetReg16(I8080_REG_H, I8080_REG_L);
}
//---------------------------------------------------------------------------
//PCHL (E9)     PC = HL
IWRAM_CODE void I8080Pchl(u8 op)
{
	TRACEOUT2("PCHL:\n");

	ST_I8080* p = &I8080;
	p->regPC = I8080GetReg16(I8080_REG_H, I8080_REG_L);
}
//---------------------------------------------------------------------------
//HLT (76)
IWRAM_CODE void I8080Hlt(u8 op)
{
	TRACEOUT2("HLT:\n");

	ST_I8080* p = &I8080;

	if( !(p->isIntEnable == TRUE && p->isIntIRQ == TRUE) )
	{
		p->regPC--;
	}
}
//---------------------------------------------------------------------------
//ADD (80-87)     A = A + reg
IWRAM_CODE void I8080Add(u8 op)
{
	TRACEOUT2("ADD %d:\n", op & 7);

	u16 reg1 = (u16)I8080GetReg8(op & 7);
	u16 reg2 = (u16)I8080GetReg8(I8080_REG_A);

	I8080OpAdd(reg1, reg2, 0);
}
//---------------------------------------------------------------------------
//ADC (88-8F)    A = A + reg + Carry
IWRAM_CODE void I8080Adc(u8 op)
{
	TRACEOUT2("ADC %d:\n", op & 7);

	ST_I8080* p = &I8080;

	u16 reg1 = (u16)I8080GetReg8(op & 7);
	u16 reg2 = (u16)I8080GetReg8(I8080_REG_A);
	u16 c    = (u16)(p->flgPSW & I8080_FLG_CARRY);

	I8080OpAdd(reg1, reg2, c);
}
//---------------------------------------------------------------------------
//ADI and ACI (C6,CE)
IWRAM_CODE void I8080Adi(u8 op)
{
	ST_I8080* p = &I8080;
	u16 c;

	if(op == 0xce)
	{
		TRACEOUT2("ACI:\n");
		c = p->flgPSW & I8080_FLG_CARRY;
	}
	else
	{
		TRACEOUT2("ADI:\n");
		c = 0;
	}

	u16 reg1 = I8080GetAtPC();
	u16 reg2 = I8080GetReg8(I8080_REG_A);

	I8080OpAdd(reg1, reg2, c);
}
//---------------------------------------------------------------------------
//SUB (90-97)
IWRAM_CODE void I8080Sub(u8 op)
{
	TRACEOUT2("SUB %d:\n", op & 7);

	u16 reg1 = (u16)I8080GetReg8(op & 7);
	u16 reg2 = (u16)I8080GetReg8(I8080_REG_A);
	u8  val  = I8080OpSub(reg1, reg2, 0);

	I8080SetReg8(I8080_REG_A, val);
}
//---------------------------------------------------------------------------
//SBB (98-9F)
IWRAM_CODE void I8080Sbb(u8 op)
{
	ST_I8080* p = &I8080;

	TRACEOUT2("SBB %d:\n", op & 7);

	u16 reg1 = (u16)I8080GetReg8(op & 7);
	u16 reg2 = (u16)I8080GetReg8(I8080_REG_A);
	u8  val  = I8080OpSub(reg1, reg2, p->flgPSW & I8080_FLG_CARRY);

	I8080SetReg8(I8080_REG_A, val);
}
//---------------------------------------------------------------------------
//SUI and SBI (D6,DE)
IWRAM_CODE void I8080Sbi(u8 op)
{
	ST_I8080* p = &I8080;
	u16 c;

	if(op == 0xde)
	{
		TRACEOUT2("SBI:\n");
		c = p->flgPSW & I8080_FLG_CARRY;
	}
	else
	{
		TRACEOUT2("SUI:\n");
		c = 0;
	}

	u16 reg1 = (u16)I8080GetAtPC();
	u16 reg2 = (u16)I8080GetReg8(I8080_REG_A);
	u8  val  = I8080OpSub(reg1, reg2, c);

	I8080SetReg8(I8080_REG_A, val);
}
//---------------------------------------------------------------------------
//CMP (B8-BF)
IWRAM_CODE void I8080Cmp(u8 op)
{
	TRACEOUT2("CMP %d:\n", op & 7);

	u16 reg1 = (u16)I8080GetReg8(op & 7);
	u16 reg2 = (u16)I8080GetReg8(I8080_REG_A);

	I8080OpSub(reg1, reg2, 0);
}
//---------------------------------------------------------------------------
//CPI (FE)
IWRAM_CODE void I8080Cpi(u8 op)
{
	TRACEOUT2("CPI:\n");

	u16 reg1 = (u16)I8080GetAtPC();
	u16 reg2 = (u16)I8080GetReg8(I8080_REG_A);

	I8080OpSub(reg1, reg2, 0);
}
//---------------------------------------------------------------------------
//ANA (A0-A7)
IWRAM_CODE void I8080Ana(u8 op)
{
	TRACEOUT2("ANA %d:\n", op & 7);

	ST_I8080* p = &I8080;

	u8 t = p->reg[I8080_REG_A];
	p->reg[I8080_REG_A] = t & I8080GetReg8(op & 7);

	I8080OpSetFlag(t);
	p->flgPSW &= ~I8080_FLG_CARRY;
}
//---------------------------------------------------------------------------
//ANI (E6)
IWRAM_CODE void I8080Ani(u8 op)
{
	TRACEOUT2("ANI:\n");

	ST_I8080* p = &I8080;

	u8 t = p->reg[I8080_REG_A] & I8080GetAtPC();
	p->reg[I8080_REG_A] = t;

	I8080OpSetFlag(t);
	p->flgPSW &= ~I8080_FLG_CARRY;
}
//---------------------------------------------------------------------------
//XRA (A8-AF)
IWRAM_CODE void I8080Xra(u8 op)
{
	TRACEOUT2("XRA %d:\n", op & 7);

	ST_I8080* p = &I8080;

	u8 t = p->reg[I8080_REG_A] ^ I8080GetReg8(op & 7);
	p->reg[I8080_REG_A] = t;

	I8080OpSetFlag(t);
	p->flgPSW &= ~I8080_FLG_CARRY;
}
//---------------------------------------------------------------------------
//XRI (EE)
IWRAM_CODE void I8080Xri(u8 op)
{
	TRACEOUT2("XRI:\n");

	ST_I8080* p = &I8080;

	u8 t = p->reg[I8080_REG_A] ^ I8080GetAtPC();
	p->reg[I8080_REG_A] = t;

	I8080OpSetFlag(t);
	p->flgPSW &= ~I8080_FLG_CARRY;
}
//---------------------------------------------------------------------------
//ORA (B0-B7)
IWRAM_CODE void I8080Ora(u8 op)
{
	TRACEOUT2("ORA %d:\n", op & 7);

	ST_I8080* p = &I8080;

	u8 t = p->reg[I8080_REG_A] | I8080GetReg8(op & 7);
	p->reg[I8080_REG_A] = t;

	I8080OpSetFlag(t);
	p->flgPSW &= ~I8080_FLG_CARRY;
}
//---------------------------------------------------------------------------
//ORI (F6)
IWRAM_CODE void I8080Ori(u8 op)
{
	TRACEOUT2("ORI:\n");

	ST_I8080* p = &I8080;

	u8 t = p->reg[I8080_REG_A] | I8080GetAtPC();
	p->reg[I8080_REG_A] = t;

	I8080OpSetFlag(t);
	p->flgPSW &= ~I8080_FLG_CARRY;
}
//---------------------------------------------------------------------------
//PUSH (C5,D5,E5,F5)
IWRAM_CODE void I8080Push(u8 op)
{
	ST_I8080* p = &I8080;

	switch(op)
	{
	case 0xc5:
		TRACEOUT2("PUSH B:\n");
		I8080MemWrite(p->regSP-1, p->reg[I8080_REG_B]);
		I8080MemWrite(p->regSP-2, p->reg[I8080_REG_C]);
		break;

	case 0xd5:
		TRACEOUT2("PUSH D:\n");
		I8080MemWrite(p->regSP-1, p->reg[I8080_REG_D]);
		I8080MemWrite(p->regSP-2, p->reg[I8080_REG_E]);
		break;

	case 0xe5:
		TRACEOUT2("PUSH H:\n");
		I8080MemWrite(p->regSP-1, p->reg[I8080_REG_H]);
		I8080MemWrite(p->regSP-2, p->reg[I8080_REG_L]);
		break;

	case 0xf5:
		TRACEOUT2("PUSH PSW:\n");
		I8080MemWrite(p->regSP-1, p->reg[I8080_REG_A]);
		I8080MemWrite(p->regSP-2, p->flgPSW);
		break;

	default:
		TRACEOUT2("PUSH Err:\n");
		return;
	}

	p->regSP -= 2;
}
//---------------------------------------------------------------------------
//POP (C1,D1,E1,F1)
IWRAM_CODE void I8080Pop(u8 op)
{
	ST_I8080* p = &I8080;

	switch(op)
	{
	case 0xc1:
		TRACEOUT2("POP B:\n");
		p->reg[I8080_REG_C] = I8080MemRead(p->regSP+0);
		p->reg[I8080_REG_B] = I8080MemRead(p->regSP+1);
		break;

	case 0xd1:
		TRACEOUT2("POP D:\n");
		p->reg[I8080_REG_E] = I8080MemRead(p->regSP+0);
		p->reg[I8080_REG_D] = I8080MemRead(p->regSP+1);
		break;

	case 0xe1:
		TRACEOUT2("POP H:\n");
		p->reg[I8080_REG_L] = I8080MemRead(p->regSP+0);
		p->reg[I8080_REG_H] = I8080MemRead(p->regSP+1);
		break;

	case 0xf1:
		TRACEOUT2("POP PSW:\n");
		p->flgPSW           = I8080MemRead(p->regSP+0);
		p->reg[I8080_REG_A] = I8080MemRead(p->regSP+1);
		break;

	default:
		TRACEOUT2("POP Err:\n");
		return;
	}

	p->regSP += 2;
}
//---------------------------------------------------------------------------
//JMP (C2,C3,CA,D2,DA,E2,EA,F2,FA)
IWRAM_CODE void I8080Jmp(u8 op)
{
	ST_I8080* p = &I8080;
	u16 addr = I8080GetAtPC() | (I8080GetAtPC() << 8);

	switch(op)
	{
	case 0xc3:
		TRACEOUT2("JMP:\n");
		p->regPC = addr;
		break;

	case 0xc2:
		TRACEOUT2("JNZ:\n");
		if( !(p->flgPSW & I8080_FLG_ZERO) )
		{
			p->regPC = addr;
		}
		break;

	case 0xca:
		TRACEOUT2("JZ:\n");
		if(p->flgPSW & I8080_FLG_ZERO)
		{
			p->regPC = addr;
		}
		break;

	case 0xd2:
		TRACEOUT2("JNC:\n");
		if( !(p->flgPSW & I8080_FLG_CARRY) )
		{
			p->regPC = addr;
		}
		break;

	case 0xda:
		TRACEOUT2("JC:\n");
		if(p->flgPSW & I8080_FLG_CARRY)
		{
			p->regPC = addr;
		}
		break;

	case 0xe2:
		TRACEOUT2("JPO:\n");
		if( !(p->flgPSW & I8080_FLG_PARITY) )
		{
			p->regPC = addr;
		}
		break;

	case 0xea:
		TRACEOUT2("JPE:\n");
		if(p->flgPSW & I8080_FLG_PARITY)
		{
			p->regPC = addr;
		}
		break;

	case 0xf2:
		TRACEOUT2("JP:\n");
		if( !(p->flgPSW & I8080_FLG_SIGN) )
		{
			p->regPC = addr;
		}
		break;

	case 0xfa:
		TRACEOUT2("JM:\n");
		if(p->flgPSW & I8080_FLG_SIGN)
		{
			p->regPC = addr;
		}
		break;

	default:
		TRACEOUT2("JUMP Err:\n");
		return;
	}
}
//---------------------------------------------------------------------------
//CALL (C4,CC,CD,D4,DC,E4,EC,F4,FC)
IWRAM_CODE void I8080Call(u8 op)
{
	ST_I8080* p = &I8080;

	switch(op)
	{
	case 0xcd:
		TRACEOUT2("CALL:\n");
		break;

	case 0xc4:
		TRACEOUT2("CNZ:\n");
		if(p->flgPSW & I8080_FLG_ZERO)
		{
			return;
		}
		break;

	case 0xcc:
		TRACEOUT2("CZ:\n");
		if( !(p->flgPSW & I8080_FLG_ZERO) )
		{
			return;
		}
		break;

	case 0xd4:
		TRACEOUT2("CNC:\n");
		if(p->flgPSW & I8080_FLG_CARRY)
		{
			return;
		}
		break;

	case 0xdc:
		TRACEOUT2("CC:\n");
		if ( !(p->flgPSW & I8080_FLG_CARRY) )
		{
			return;
		}
		break;

	case 0xe4:
		TRACEOUT2("CPO:\n");
		if(p->flgPSW & I8080_FLG_PARITY)
		{
			return;
		}
		break;

	case 0xec:
		TRACEOUT2("CPE:\n");
		if( !(p->flgPSW & I8080_FLG_PARITY) )
		{
			return;
		}
		break;

	case 0xf4:
		TRACEOUT2("CP:\n");
		if(p->flgPSW & I8080_FLG_SIGN)
		{
			return;
		}
		break;

	case 0xfc:
		TRACEOUT2("CM:\n");
		if( !(p->flgPSW & I8080_FLG_SIGN) )
		{
			return;
		}
		break;

	default:
		TRACEOUT2("CALL Err:\n");
		return;
	}

	u16 addr = I8080GetAtPC() | (I8080GetAtPC() << 8);

	I8080MemWrite(p->regSP-1, (p->regPC >> 8) & 0xff);
	I8080MemWrite(p->regSP-2,  p->regPC       & 0xff);
	p->regSP -= 2;
	p->regPC  = addr;
}
//---------------------------------------------------------------------------
//return (C9,C0,C8,D0,D8,E0,E8,F0,F8)
IWRAM_CODE void I8080Ret(u8 op)
{
	ST_I8080* p = &I8080;

	switch(op)
	{
	case 0xc9:
		TRACEOUT2("RET:\n");
		break;

	case 0xc0:
		TRACEOUT2("RNZ:\n");
		if (p->flgPSW & I8080_FLG_ZERO)
		{
			return;
		}
		break;

	case 0xc8:
		TRACEOUT2("RZ:\n");
		if( !(p->flgPSW & I8080_FLG_ZERO) )
		{
			return;
		}
		break;

	case 0xd0:
		TRACEOUT2("RNC:\n");
		if(p->flgPSW & I8080_FLG_CARRY)
		{
			return;
		}
		break;

	case 0xd8:
		TRACEOUT2("RC:\n");
		if( !(p->flgPSW & I8080_FLG_CARRY) )
		{
			return;
		}
		break;

	case 0xe0:
		TRACEOUT2("RPO:\n");
		if(p->flgPSW & I8080_FLG_PARITY)
		{
			return;
		}
		break;

	case 0xe8:
		TRACEOUT2("RPE:\n");
		if( !(p->flgPSW & I8080_FLG_PARITY) )
		{
			return;
		}
		break;

	case 0xf0:
		TRACEOUT2("RP:\n");
		if(p->flgPSW & I8080_FLG_SIGN)
		{
			return;
		}
		break;

	case 0xf8:
		TRACEOUT2("RM:\n");
		if( !(p->flgPSW & I8080_FLG_SIGN) )
		{
			return;
		}
		break;

	default:
		TRACEOUT2("return Err:\n");
		return;
	}

	p->regPC  = I8080MemRead(p->regSP+0) | (I8080MemRead(p->regSP+1) << 8);
	p->regSP += 2;
}
//---------------------------------------------------------------------------
//IN and OUT (D3,DB)
IWRAM_CODE void I8080InOut(u8 op)
{
	ST_I8080* p = &I8080;
	u8 addr = I8080GetAtPC();

	if(op == 0xd3)
	{
		TRACEOUT2("OUT:\n");
		I8255Out(addr, p->reg[I8080_REG_A]);
	}
	else
	{
		TRACEOUT2("IN:\n");
		p->reg[I8080_REG_A] = I8255In(addr);
	}
}
//---------------------------------------------------------------------------
//DI and EI (F3,FB)
//荞݃Cl[uMtO̕ύX܂
IWRAM_CODE void I8080DiEi(u8 op)
{
	ST_I8080* p = &I8080;

	if(op == 0xf3)
	{
		TRACEOUT2("DI:\n");
		p->isIntEnable = FALSE;
	}
	else
	{
		TRACEOUT2("EI:\n");
		p->isIntEnable = TRUE;
	}
}
//---------------------------------------------------------------------------
//RST (C7,CF,D7,DF,E7,EF,F8,FF)
IWRAM_CODE void I8080Rst(u8 op)
{
	u16 addr;

	switch(op)
	{
	case 0xc7:
		TRACEOUT2("RST 0:\n");
		addr = 0;
		break;

	case 0xcf:
		TRACEOUT2("RST 1:\n");
		addr = 8;
		break;

	case 0xd7:
		TRACEOUT2("RST 2:\n");
		addr = 0x10;
		break;

	case 0xdf:
		TRACEOUT2("RST 3:\n");
		addr = 0x18;
		break;

	case 0xe7:
		TRACEOUT2("RST 4:\n");
		addr = 0x20;
		break;

	case 0xef:
		TRACEOUT2("RST 5:\n");
		addr = 0x28;
		break;

	case 0xf7:
		TRACEOUT2("RST 6:\n");
		addr = 0x30;
		break;

	case 0xff:
		TRACEOUT2("RST 7:\n");
		addr = 0x38;
		break;

	default:
		TRACEOUT2("RST Err:\n");
		return;
	}

	ST_I8080* p = &I8080;

	I8080MemWrite(p->regSP-1, (p->regPC >> 8) & 0xff);
	I8080MemWrite(p->regSP-2,  p->regPC       & 0xff);
	p->regSP -= 2;
	p->regPC  = addr;
}
