
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "ad_arm.h"
#include "gbfs.h"
#include "lib.h"
#include "rand.h"

#include "screen.h"
#include "tk80.h"
#include "i8080.h"
#include "i8255.h"

#include "main.h"


//#include "header\font\BGFont.h"


//Mode3
//bg  
//Spr {^AXCb`

//---------------------------------------------------------------------------
u8 State;

//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:  StateInit();  break;
			case STATE_RESET: StateReset(); break;
			case STATE_DRAW:  StateDraw();  break;
			case STATE_KEY:   StateKey();   break;
			case STATE_CPU:   StateCpu();   break;
			case STATE_MENU:  StateMenu();  break;

			default:
				DebugMessage("[Err] main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitLib();
	InitKey();
	InitIRQ();
	InitFont();
	InitSprite();

	SetMode(MODE_3 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);
	InitGBFS();
	InitLSI(NULL);

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitLib()
{
	LibInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER2;

	REG_TM2D     = 65536 - 2621;			// (((16*1024*1024) / 64) * 10) / 1000
	REG_TM2CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitFont()
{
	ST_FONT_SRC half;		//p

	half.pDat   = (u8*)&mplus_s10rBitmap;
	half.pSheet = (u16*)&mplus_sfnt_txt;
	half.imgCx  = MPLUS_S10R_IMG_CX;
	half.cnt    = MPLUS_S10R_FONT_CNT;
	half.cx     = MPLUS_S10R_FONT_CX;
	half.cy     = MPLUS_S10R_FONT_CY;

	Mode3SetFont(NULL, &half);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitSprite()
{
	SpriteInit();

	SpriteSetPalData16((u16*)&sprPal);
	SpriteSetData((u16*)&sprTiles, sprTilesLen);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitGBFS()
{
	if( GBFSInit() == FALSE )
	{
		SystemError("[Err] GBFS Not Found.", NULL);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitLSI(char* pName)
{
	Tk80Init();
	I8080Init();
	I8255Init();

	I8080Reset();
	I8255Reset();

	if(pName != NULL)
	{
		Tk80LoadRam(pName);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	ScreenImgFadeIn(BLEND_MODE_DARK | BLEND_TOP_OBJ, 0, 15, 0);
	ScreenImgInit();

	Tk80DrawBasis();
	ScreenImgUpdate();
	ScreenImgFadeOut(10);

	State = STATE_DRAW;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateDraw()
{
	Tk80DrawLcd();

	WaitForVsync();
	ScreenImgUpdate();
	Tk80DrawButton();
	Tk80DrawCursor();

	State = STATE_CPU;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateCpu()
{
	I8080Exec();

	State = STATE_KEY;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateKey()
{
	State = (Tk80CursorKey() == TRUE) ? STATE_DRAW : STATE_MENU;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateMenu()
{
	SystemMenu();

	State = STATE_RESET;
}









//---------------------------------------------------------------------------
EWRAM_CODE void SystemMenu()
{
	u16  maxCnt = GBFSGetFileCnt();
	u16  cnt    = 0;
	u16  key;
	char fname[14];

	GBFSGetFilePointer2(cnt);		//|C^͎擾At@CĂ܂
	_Strcpy((u8*)fname, (u8*)GBFSGetFileName());
	SystemMessage("Select File: ", fname);

	for(;;)
	{
RE_KEY:
		WaitForVsync();
		key = KeyGet();

		if(key == KEY_NONE)
		{
			continue;
		}

		if(key & KEY_A)
		{
			break;
		}
		else if(key & KEY_LEFT)
		{
			if(cnt == 0) continue;
			cnt--;

		}
		else if(key & KEY_RIGHT)
		{
			if(cnt+1 >= maxCnt) continue;
			cnt++;
		}

		GBFSGetFilePointer2(cnt);
		_Strcpy((u8*)fname, (u8*)GBFSGetFileName());
		SystemMessage("Select File: ", fname);
	}

	if( Tk80IsRamData(fname) == FALSE )
	{
		goto RE_KEY;
	}

	ScreenImgFadeIn(BLEND_MODE_DARK | BLEND_TOP_OBJ, 0, 15, 20);
	InitLSI(fname);
}


//---------------------------------------------------------------------------
EWRAM_CODE void SystemMessage(char* msg1, char* msg2)
{
	u16 cx  = 7;
	u16 cy  = 11;
	u16 len = _Strlen((u8*)msg1) * cx;

	Mode3DrawFillBox(0, SCREEN_CY-cy, 240, cy, COLOR_BLACK);

	if(msg1 != NULL) Mode3DrawFontStr(  0, SCREEN_CY-cy, (u8*)msg1, COLOR_WHITE);
	if(msg2 != NULL) Mode3DrawFontStr(len, SCREEN_CY-cy, (u8*)msg2, COLOR_WHITE);

	WaitForVsync();
	ScreenImgUpdate();
}
//---------------------------------------------------------------------------
EWRAM_CODE void SystemError(char* msg1, char* msg2)
{
	u16 cy = 11;

	ScreenImgFadeOut(0);
	LibMode3SetScreenBuffer((u16*)VideoBuffer);

	if(msg1 != NULL) Mode3DrawFontStr(0,  0, (u8*)msg1, COLOR_WHITE);
	if(msg2 != NULL) Mode3DrawFontStr(0, cy, (u8*)msg2, COLOR_WHITE);

	for(;;){}
}
