
#include "inc.h"
#include "gbfs.h"
#include "lib.h"
#include "minix.h"
#include "div_arm.h"

#include "main.h"
#include "screen.h"
#include "i8080.h"
#include "i8255.h"
#include "tk80.h"

//---------------------------------------------------------------------------
//GBANɕKvȃt@CXg
ROM_DATA ST_TK80_FILE_LIST Tk80FileList[] = {
	// img
	{"bg.img"      },
	{"led.img"     },
	// bin
	{"rom.bin"     },
	// tk8
	{"Flipflop.tk8"},
	{"Mogura.tk8"  },
	{"Moo.tk8"     },
	{"Oddcpls.tk8" },
	{"Slot.tk8"    },
	{"TIMER.TK8"   },
	{"Wildsevn.tk8"},
	{""            },
};

ROM_DATA ST_TK80_BUTTON_SPR Tk80ButtonSpr[] = {
	{ 125, 512, '0', "0"},					//0x00
	{ 124, 514, '1', "1"},
	{ 123, 516, '2', "2"},
	{ 122, 518, '3', "3"},
	{ 121, 520, '4', "4"},
	{ 120, 522, '5', "5"},
	{ 119, 524, '6', "6"},
	{ 118, 526, '7', "7"},
	{ 117, 528, '8', "8"},
	{ 116, 530, '9', "9"},
	{ 115, 532, 'a', "A"},
	{ 114, 534, 'b', "B"},
	{ 113, 536, 'c', "C"},
	{ 112, 538, 'd', "D"},
	{ 111, 540, 'e', "E"},
	{ 110, 542, 'f', "F"},
	{ 108, 576, 'R', "RUN"            },	//0x10
	{ 109, 578, 'E', "RET"            },
	{ 104, 580, 'A', "ADRS SET"       },
	{ 102, 582, 'D', "READ DECR"      },
	{ 103, 584, 'I', "READ INCR"      },
	{ 101, 586, 'W', "WRITE INCR"     },
	{ 107, 588, 'S', "STORE DATA"     },
	{ 106, 590, 'L', "LOAD DATA"      },

	{ 105, 592, 'T', "RESET"          },	//0x18
	{ 127, 640, 'P', "PROTECT SW"     },
	{ 126, 640, 'I', "SINGLE STEP SW" },
};


ROM_DATA ST_TK80_BUTTON_MAP Tk80ButtonMap = {
	TK80_BUTTON_MAP_SX, TK80_BUTTON_MAP_SY,
	{
		{ 25, -1,  0,  1,  2,  3, -1, 16, 17, 18, 19, -1 },
		{ 26, -1,  4,  5,  6,  7, -1, 20, 21, 22, 23, 24 },
		{ -1, -1,  8,  9, 10, 11, -1, -1, -1, -1, -1, -1 },
		{ -1, -1, 12, 13, 14, 15, -1, -1, -1, -1, -1, -1 },
	},
};
//---------------------------------------------------------------------------
ST_TK80        Tk80;
ST_TK80_LED    Tk80Led;
ST_TK80_KEY    Tk80Key;
ST_TK80_CURSOR Tk80Cursor;

extern ST_I8080 I8080;				//i8080.c
extern ST_I8255 I8255;				//i8255.c
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80Init()
{
	_Memset((u8*)&Tk80,       0x00, sizeof(ST_TK80));
	_Memset((u8*)&Tk80Led,    0x00, sizeof(ST_TK80_LED));
	_Memset((u8*)&Tk80Key,    0x00, sizeof(ST_TK80_KEY));
	_Memset((u8*)&Tk80Cursor, 0x00, sizeof(ST_TK80_CURSOR));

	Tk80InitFileList();

	Tk80InitLed();
	Tk80InitKey();
	Tk80InitCursor();
	Tk80InitButton();

	Tk80LoadRom();
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80InitFileList()
{
	ST_TK80_FILE_LIST* p = &Tk80FileList[0];

	while(p->name[0] != '\0')
	{
		if( GBFSGetFilePointer(p->name) == NULL )
		{
			SystemError("[Err] File Not Found.", p->name);
		}

		p++;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80InitLed()
{
	ST_TK80_LED* p = &Tk80Led;

	p->sx = TK80_LED_SX;
	p->sy = TK80_LED_SY;

	u16 i;
	for(i=0; i<TK80_LED_CNT; i++)
	{
		p->var[i] = 0;
	}

	p->pImg = (u16*)GBFSGetFilePointer("led.img");
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80InitKey()
{
	ST_TK80_KEY* p = &Tk80Key;

	//L[}gNX̏iŌ3̓_~[łj
	//bit 12-14: PC4-PC6
	//bit 0-7  : PA0-PA7
	const u16 matrix[] = {
		0x10fe, 0x10fd, 0x10fb, 0x10f7, 0x10ef, 0x10df, 0x10bf, 0x107f,
		0x20fe, 0x20fd, 0x20fb, 0x20f7, 0x20ef, 0x20df, 0x20bf, 0x207f,
		0x40fe, 0x40fd, 0x40fb, 0x40f7, 0x40ef, 0x40df, 0x40bf, 0x407f,
		0x0000, 0x0000, 0x0000,
	};

	u16 i;
	for(i=0; i<TK80_KEY_CNT; i++)
	{
		p->matrix[i] = matrix[i];
		p->isDown[i] = FALSE;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80InitButton()
{
	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_BUTTON_SPR* s;

	u16 x, y;
	u16 num;

	for(y=0; y<TK80_BUTTON_MAP_CY; y++)
	{
		for(x=0; x<TK80_BUTTON_MAP_CX; x++)
		{
			if(m->btnSpr[y][x] == -1)
			{
				continue;
			}

			num = m->btnSpr[y][x];
			s   = &Tk80ButtonSpr[num];

			SpriteSetSize(s->sprNo, SP_SIZE_16, SP_SQUARE, SP_COLOR_16);
			SpriteSetChr (s->sprNo, s->tileNo);
			SpriteMove   (s->sprNo, SCREEN_CX, SCREEN_CY);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80InitCursor()
{
	ST_TK80_CURSOR* p = &Tk80Cursor;

	p->vx     = 0;
	p->vy     = 0;
	p->sprNo  = TK80_CURSOR_SPRNO;
	p->tileNo = TK80_CURSOR_TILENO;

	SpriteSetSize(p->sprNo, SP_SIZE_16, SP_SQUARE, SP_COLOR_16);
	SpriteSetChr (p->sprNo, p->tileNo);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80LoadRom()
{
	u8* pRom = GBFSGetFilePointer("rom.bin");
	u32 size = GBFSGetFileSize();

	if(size != TK80_ROM_SIZE)
	{
		SystemError("[Err] ROM File Size.", "rom.bin");
	}

	_Memcpy(Tk80.rom, pRom, TK80_ROM_SIZE);

	//WAITJE^̏Cpb`
	Tk80.rom[0x2de] = 0x03;
	Tk80.rom[0x2eb] = 0x07;
	Tk80.rom[0x2f0] = 0x10;
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80LoadRam(char* pName)
{
	u8* pRam = GBFSGetFilePointer(pName);
	u32 size = GBFSGetFileSize();

	if(size != TK80_RAM_SIZE)
	{
		SystemError("[Err] RAM File Size.", pName);
	}

	_Memcpy(Tk80.ram, pRam, TK80_RAM_SIZE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80DrawBasis()
{
	u16* pBg = (u16*)GBFSGetFilePointer("bg.img");
	ScreenImgLoad(pBg);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80DrawButton()
{
	ST_TK80_KEY*        k = &Tk80Key;
	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_BUTTON_SPR* s;

	u16 sx, sy;
	u16 x,  y;
	u16 add;
	u8  num;

	for(y=0; y<TK80_BUTTON_MAP_CY; y++)
	{
		for(x=0; x<TK80_BUTTON_MAP_CX; x++)
		{
			if(m->btnSpr[y][x] == -1)
			{
				continue;
			}

			num = m->btnSpr[y][x];
			s   = &Tk80ButtonSpr[num];
			add = (k->isDown[num] == TRUE) ? 2 : 0;
			sx  = m->sx + (x * TK80_BUTTON_CX) + add;
			sy  = m->sy + (y * TK80_BUTTON_CY) + add;

			SpriteMove(s->sprNo, sx, sy);


			if(s->chr == 'P' || s->chr == 'I')
			{
				SpriteSetChr(s->sprNo, s->tileNo + add);
			}
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80DrawCursor()
{
	ST_TK80_CURSOR*     c = &Tk80Cursor;
	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_KEY*        k = &Tk80Key;

	u16 pos = m->btnSpr[c->vy][c->vx];
	u16 add = (k->isDown[pos] == TRUE) ? 2 : 0;

	s16 sx  = m->sx + (c->vx * TK80_BUTTON_CX) + add;
	s16 sy  = m->sy + (c->vy * TK80_BUTTON_CY) + add;

	SpriteMove(c->sprNo, sx, sy);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80DrawLcd()
{
	ST_I8255*    w = &I8255;
	ST_TK80_LED* p = &Tk80Led;

	u16  dx, dy;
	u8   var;
	u16  i;

	for(i=0; i<TK80_LED_CNT; i++)
	{
		if(w->portC & 0x80)
		{
			var = I8080MemRead(0x83f8 + i);
		}
		else
		{
			var = 0;
		}

		if(p->var[i] != var)
		{
			p->var[i] = var;
			dx        = _UMod(var, 16) * 28;
			dy        = _UDiv(var, 16) * 40;

			Mode3DrawImage2(1+30*i, 2, LED_CX, LED_CY, dx, dy, LED_IMG_CX, p->pImg);
		}
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80DrawMsg()
{
	ST_TK80_CURSOR*     c = &Tk80Cursor;
	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_BUTTON_SPR* s = &Tk80ButtonSpr[0];

	u16 pos = m->btnSpr[c->vy][c->vx];

	SystemMessage("KEY: ", s[pos].name);
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80UpdateKey()
{
	ST_I8255*    p = &I8255;
	ST_TK80_KEY* k = &Tk80Key;

	u8  x = 0xff;
	u16 c = (~(u16)p->portC << 8) & 0x7000;
	u16 i;

	for(i=0; i<TK80_KEY_CNT-3; i++)			//-3́A_~[̃{^ł
	{
		if((k->isDown[i] == TRUE) && (k->matrix[i] & c))
		{
			x &= (u8)(k->matrix[i] & 0xff);
		}
	}

//	TRACEOUT("SCAN CODE = %02x/%02x\n", c, x);
	p->portA = x;
}




//---------------------------------------------------------------------------
EWRAM_CODE bool Tk80CursorKey()
{
	u16 key = KeyGet();
	u16 now = KeyGetNow();

	if(now & KEY_SELECT)
	{
		return FALSE;
	}

	if(now & KEY_A)
	{
		Tk80CursorDown();
	}
	else if( !(now & KEY_A) )
	{
		Tk80CursorUp();
	}

	if(key & KEY_MOVE)
	{
		if( Tk80CursorIsMove(key) == TRUE )
		{
			Tk80CursorMove(key);
		}
	}

	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80CursorUp()
{
	ST_TK80_CURSOR*     c = &Tk80Cursor;
	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_KEY*        k = &Tk80Key;

	u16 num = m->btnSpr[c->vy][c->vx];
	k->isDown[num] = FALSE;
	c->isDown      = FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80CursorDown()
{
	ST_TK80_CURSOR*     c = &Tk80Cursor;
	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_KEY*        k = &Tk80Key;

	u16 num = m->btnSpr[c->vy][c->vx];
	k->isDown[num] = TRUE;
	c->isDown      = TRUE;

	//RESET
	if(Tk80ButtonSpr[num].chr == 'T')
	{
		I8080.isReset = TRUE;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 Tk80CursorGetChr()
{
	ST_TK80_CURSOR*     c = &Tk80Cursor;
	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_BUTTON_SPR* s = &Tk80ButtonSpr[0];

	s8 btnSpr = m->btnSpr[c->vy][c->vx];

	return s[ btnSpr ].chr;
}
//---------------------------------------------------------------------------
EWRAM_CODE void Tk80CursorMove(u16 key)
{
	s16 sx = 0;
	s16 sy = 0;

	if(key & KEY_DOWN)  sy++;
	if(key & KEY_UP  )  sy--;
	if(key & KEY_RIGHT) sx++;
	if(key & KEY_LEFT)  sx--;

	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_CURSOR*     c = &Tk80Cursor;

	s16 vx = c->vx;
	s16 vy = c->vy;

	do {
		vx += sx;
		vy += sy;

	} while(m->btnSpr[vy][vx] == -1);

	c->vx = vx;
	c->vy = vy;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool Tk80CursorIsMove(u16 key)
{
	s16 sx = 0;
	s16 sy = 0;

	if(key & KEY_DOWN)  sy++;
	if(key & KEY_UP  )  sy--;
	if(key & KEY_RIGHT) sx++;
	if(key & KEY_LEFT)  sx--;

	if(sx == 0 && sy == 0)
	{
		return FALSE;
	}

	ST_TK80_BUTTON_MAP* m = &Tk80ButtonMap;
	ST_TK80_CURSOR*     c = &Tk80Cursor;

	s16 vx = c->vx;
	s16 vy = c->vy;

	if(c->isDown == TRUE)
	{
		return FALSE;
	}

	do {
		vx += sx;
		vy += sy;

		if(vx < 0 || TK80_BUTTON_MAP_CX <= vx)
		{
			return FALSE;
		}

		if(vy < 0 || TK80_BUTTON_MAP_CY <= vy)
		{
			return FALSE;
		}

	} while(m->btnSpr[vy][vx] == -1);


	return TRUE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool Tk80IsRamData(char* pName)
{
	u16 len = _Strlen((u8*)pName);

	if(len <= 4)
	{
		return FALSE;
	}


	u8 buf[4];

	buf[0] = pName[len-4];
	buf[1] = _Toupper(pName[len-3]);
	buf[2] = _Toupper(pName[len-2]);
	buf[3] = pName[len-1];

	if(buf[0] != '.') return FALSE;
	if(buf[1] != 'T') return FALSE;
	if(buf[2] != 'K') return FALSE;
	if(buf[3] != '8') return FALSE;

	return TRUE;
}
