
#ifndef __TK80_H__
#define __TK80_H__
#ifdef __cplusplus
extern "C" {
#endif


//---------------------------------------------------------------------------
#define TK80_RAM_SIZE			1024
#define TK80_ROM_SIZE			768
#define TK80_LED_CNT			8

#define TK80_BUTTON_CX			16
#define TK80_BUTTON_CY			16
#define TK80_BUTTON_MAP_SX		24
#define TK80_BUTTON_MAP_SY		72
#define TK80_BUTTON_MAP_CX		12
#define TK80_BUTTON_MAP_CY		4

#define TK80_LED_SX				0
#define TK80_LED_SY				0
#define TK80_LED_CX				30
#define TK80_LED_CY				44
#define TK80_LED_CNT			8

#define TK80_KEY_CNT			27

#define TK80_CURSOR_SPRNO		0
#define TK80_CURSOR_TILENO		644
//---------------------------------------------------------------------------

typedef struct {
	char name[13];
} ST_TK80_FILE_LIST;


//----
typedef struct {
	u8   ram[TK80_RAM_SIZE];		//1024oCgRAM (8000-83FF)
	u8   rom[TK80_ROM_SIZE];		// 768oCgROM (0000-02FF)
} ST_TK80;


//----
typedef struct {
	s16  sx;
	s16  sy;
	u16* pImg;
	u8   var[TK80_LED_CNT];
} ST_TK80_LED;


//----
typedef struct {
	u16  matrix[TK80_KEY_CNT];
	bool isDown[TK80_KEY_CNT];
} ST_TK80_KEY;


//----
typedef struct {
	u16  sprNo;
	u16  tileNo;
	u8   chr;
	char name[16];
} ST_TK80_BUTTON_SPR;

typedef struct {
	s16  sx;						//N_ʒu
	s16  sy;
	s8   btnSpr[TK80_BUTTON_CX][TK80_BUTTON_CY];
} ST_TK80_BUTTON_MAP;


//----
typedef struct {
	s16  vx;						//z̈ʒu
	s16  vy;
	u16  sprNo;
	u16  tileNo;
	bool isDown;
} ST_TK80_CURSOR;


//---------------------------------------------------------------------------
EWRAM_CODE void Tk80Init();
EWRAM_CODE void Tk80InitFileList();
EWRAM_CODE void Tk80InitLed();
EWRAM_CODE void Tk80InitKey();
EWRAM_CODE void Tk80InitCursor();
EWRAM_CODE void Tk80InitButton();

EWRAM_CODE void Tk80LoadRom();
EWRAM_CODE void Tk80LoadRam(char* pName);

EWRAM_CODE void Tk80DrawBasis();
EWRAM_CODE void Tk80DrawButton();
EWRAM_CODE void Tk80DrawCursor();
EWRAM_CODE void Tk80DrawLcd();
EWRAM_CODE void Tk80DrawMsg();

EWRAM_CODE void Tk80UpdateKey();

EWRAM_CODE bool Tk80IsRamData(char* pName);

//----
EWRAM_CODE bool Tk80CursorKey();
EWRAM_CODE void Tk80CursorUp();
EWRAM_CODE void Tk80CursorDown();
EWRAM_CODE u8   Tk80CursorGetChr();
EWRAM_CODE void Tk80CursorMove(u16 key);
EWRAM_CODE bool Tk80CursorIsMove(u16 key);



#ifdef __cplusplus
}
#endif
#endif
